package com.sun.jmx.interceptor;

import javax.management.Notification;
import com.sun.jmx.mbeanserver.ModifiableClassLoaderRepository;
import java.security.AccessControlContext;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.ProtectionDomain;
import javax.management.MBeanTrustPermission;
import javax.management.MBeanPermission;
import javax.management.loading.ClassLoaderRepository;
import javax.management.OperationsException;
import java.io.ObjectInputStream;
import javax.management.QueryEval;
import com.sun.jmx.mbeanserver.NamedObject;
import javax.management.MBeanServerNotification;
import javax.management.IntrospectionException;
import javax.management.JMRuntimeException;
import javax.management.MBeanInfo;
import javax.management.NotificationEmitter;
import javax.management.ListenerNotFoundException;
import javax.management.NotificationBroadcaster;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import com.sun.jmx.mbeanserver.DynamicMBean2;
import javax.management.RuntimeMBeanException;
import javax.management.RuntimeErrorException;
import javax.management.InvalidAttributeValueException;
import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import com.sun.jmx.mbeanserver.Util;
import java.util.ArrayList;
import java.util.Iterator;
import javax.management.QueryExp;
import javax.management.MBeanRegistration;
import javax.management.DynamicMBean;
import com.sun.jmx.mbeanserver.Introspector;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import javax.management.RuntimeOperationsException;
import com.sun.jmx.remote.util.EnvHelp;
import javax.management.InstanceNotFoundException;
import javax.management.NotCompliantMBeanException;
import javax.management.MBeanException;
import javax.management.MBeanRegistrationException;
import javax.management.InstanceAlreadyExistsException;
import javax.management.ReflectionException;
import javax.management.ObjectInstance;
import java.util.HashSet;
import javax.management.ObjectName;
import java.util.Set;
import java.lang.ref.WeakReference;
import java.util.WeakHashMap;
import com.sun.jmx.mbeanserver.Repository;
import javax.management.MBeanServerDelegate;
import javax.management.MBeanServer;
import com.sun.jmx.mbeanserver.MBeanInstantiator;

public class DefaultMBeanServerInterceptor implements MBeanServerInterceptor
{
    private final transient MBeanInstantiator instantiator;
    private transient MBeanServer server;
    private final transient MBeanServerDelegate delegate;
    private final transient Repository repository;
    private final transient WeakHashMap<DefaultMBeanServerInterceptor.ListenerWrapper, WeakReference<DefaultMBeanServerInterceptor.ListenerWrapper>> listenerWrappers;
    private final String domain;
    private final Set<ObjectName> beingUnregistered;
    
    public DefaultMBeanServerInterceptor(MBeanServer server, MBeanServerDelegate delegate, MBeanInstantiator instantiator, Repository repository) {
        this.server = null;
        this.listenerWrappers = new WeakHashMap();
        this.beingUnregistered = new HashSet();
        if (server == null) {
            throw new IllegalArgumentException("outer MBeanServer cannot be null");
        }
        if (delegate == null) {
            throw new IllegalArgumentException("MBeanServerDelegate cannot be null");
        }
        if (instantiator == null) {
            throw new IllegalArgumentException("MBeanInstantiator cannot be null");
        }
        if (repository == null) {
            throw new IllegalArgumentException("Repository cannot be null");
        }
        this.server = server;
        this.delegate = delegate;
        this.instantiator = instantiator;
        this.repository = repository;
        this.domain = repository.getDefaultDomain();
        return;
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException {
        return this.createMBean(s, objectName, (Object[])null, (String[])null);
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName, ObjectName objectName2) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException {
        return this.createMBean(s, objectName, objectName2, (Object[])null, (String[])null);
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName, Object[] array, String[] array2) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException {
        try {
            return this.createMBean(s, objectName, null, true, array, array2);
        }
        catch (InstanceNotFoundException ex) {
            throw (IllegalArgumentException)EnvHelp.initCause(new IllegalArgumentException("Unexpected exception: " + ex), ex);
        }
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName, ObjectName objectName2, Object[] array, String[] array2) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException {
        return this.createMBean(s, objectName, objectName2, false, array, array2);
    }
    
    private ObjectInstance createMBean(String s, ObjectName nonDefaultDomain, ObjectName nonDefaultDomain2, boolean b, Object[] params, String[] signature) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException {
        if (s == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("The class name cannot be null"), "Exception occurred during MBean creation");
        }
        if (nonDefaultDomain != null) {
            if (nonDefaultDomain.isPattern()) {
                throw new RuntimeOperationsException(new IllegalArgumentException("Invalid name->" + nonDefaultDomain.toString()), "Exception occurred during MBean creation");
            }
            nonDefaultDomain = this.nonDefaultDomain(nonDefaultDomain);
        }
        checkMBeanPermission(s, null, null, "instantiate");
        checkMBeanPermission(s, null, nonDefaultDomain, "registerMBean");
        Class<Object> theClass;
        if (b) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "createMBean", "ClassName = " + s + ", ObjectName = " + nonDefaultDomain);
            }
            theClass = this.instantiator.findClassWithDefaultLoaderRepository(s);
        }
        else if (nonDefaultDomain2 == null) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "createMBean", "ClassName = " + s + ", ObjectName = " + nonDefaultDomain + ", Loader name = null");
            }
            theClass = this.instantiator.findClass(s, this.server.getClass().getClassLoader());
        }
        else {
            nonDefaultDomain2 = this.nonDefaultDomain(nonDefaultDomain2);
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "createMBean", "ClassName = " + s + ", ObjectName = " + nonDefaultDomain + ", Loader name = " + nonDefaultDomain2);
            }
            theClass = this.instantiator.findClass(s, nonDefaultDomain2);
        }
        checkMBeanTrustPermission(theClass);
        Introspector.testCreation(theClass);
        Introspector.checkCompliance(theClass);
        Object instantiate = this.instantiator.instantiate(theClass, params, signature, this.server.getClass().getClassLoader());
        return this.registerObject(getNewMBeanClassName(instantiate), instantiate, nonDefaultDomain);
    }
    
    @Override
    public ObjectInstance registerMBean(Object o, ObjectName objectName) throws InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {
        Class<Object> class1 = o.getClass();
        Introspector.checkCompliance(class1);
        String newMBeanClassName = getNewMBeanClassName(o);
        checkMBeanPermission(newMBeanClassName, null, objectName, "registerMBean");
        checkMBeanTrustPermission(class1);
        return this.registerObject(newMBeanClassName, o, objectName);
    }
    
    private static String getNewMBeanClassName(Object o) throws NotCompliantMBeanException {
        if ((o instanceof DynamicMBean)) {
            DynamicMBean dynamicMBean = (DynamicMBean)o;
            String className;
            try {
                className = dynamicMBean.getMBeanInfo().getClassName();
            }
            catch (Exception cause) {
                NotCompliantMBeanException ex = new NotCompliantMBeanException("Bad getMBeanInfo()");
                ex.initCause(cause);
                throw ex;
            }
            if (className == null) {
                throw new NotCompliantMBeanException("MBeanInfo has null class name");
            }
            return className;
        }
        else {
            return o.getClass().getName();
        }
    }
    
    @Override
    public void unregisterMBean(ObjectName nonDefaultDomain) throws InstanceNotFoundException, MBeanRegistrationException {
        if (nonDefaultDomain == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Object name cannot be null"), "Exception occurred trying to unregister the MBean");
        }
        nonDefaultDomain = this.nonDefaultDomain(nonDefaultDomain);
        synchronized (this.beingUnregistered) {
            while (this.beingUnregistered.contains(nonDefaultDomain)) {
                try {
                    this.beingUnregistered.wait();
                }
                catch (InterruptedException e) {
                    throw new MBeanRegistrationException(e, e.toString());
                }
            }
            this.beingUnregistered.add(nonDefaultDomain);
        }
        try {
            this.exclusiveUnregisterMBean(nonDefaultDomain);
        }
        finally {
            synchronized (this.beingUnregistered) {
                this.beingUnregistered.remove(nonDefaultDomain);
                this.beingUnregistered.notifyAll();
            }
        }
        return;
    }
    
    private void exclusiveUnregisterMBean(ObjectName objectName) throws InstanceNotFoundException, MBeanRegistrationException {
        DynamicMBean mBean = this.getMBean(objectName);
        checkMBeanPermission(mBean, null, objectName, "unregisterMBean");
        if ((mBean instanceof MBeanRegistration)) {
            preDeregisterInvoke((MBeanRegistration)mBean);
        }
        DefaultMBeanServerInterceptor.ResourceContext unregisterFromRepository = this.unregisterFromRepository(getResource(mBean), mBean, objectName);
        try {
            if ((mBean instanceof MBeanRegistration)) {
                postDeregisterInvoke(objectName, (MBeanRegistration)mBean);
            }
        }
        finally {
            unregisterFromRepository.done();
        }
    }
    
    @Override
    public ObjectInstance getObjectInstance(ObjectName nonDefaultDomain) throws InstanceNotFoundException {
        nonDefaultDomain = this.nonDefaultDomain(nonDefaultDomain);
        DynamicMBean mBean = this.getMBean(nonDefaultDomain);
        checkMBeanPermission(mBean, null, nonDefaultDomain, "getObjectInstance");
        return new ObjectInstance(nonDefaultDomain, getClassName(mBean));
    }
    
    @Override
    public Set<ObjectInstance> queryMBeans(ObjectName objectName, QueryExp queryExp) {
        if (System.getSecurityManager() != null) {
            checkMBeanPermission((String)null, null, null, "queryMBeans");
            Set<ObjectInstance> queryMBeansImpl = this.queryMBeansImpl(objectName, null);
            HashSet<ObjectInstance> set = new HashSet(queryMBeansImpl.size());
            for (ObjectInstance objectInstance : queryMBeansImpl) {
                try {
                    checkMBeanPermission(objectInstance.getClassName(), null, objectInstance.getObjectName(), "queryMBeans");
                    set.add(objectInstance);
                }
                catch (SecurityException ex) {}
            }
            return this.filterListOfObjectInstances(set, queryExp);
        }
        return this.queryMBeansImpl(objectName, queryExp);
    }
    
    private Set<ObjectInstance> queryMBeansImpl(ObjectName pattern, QueryExp query) {
        return this.objectInstancesFromFilteredNamedObjects(this.repository.query(pattern, query), query);
    }
    
    @Override
    public Set<ObjectName> queryNames(ObjectName objectName, QueryExp queryExp) {
        if (System.getSecurityManager() != null) {
            checkMBeanPermission((String)null, null, null, "queryNames");
            Set<ObjectInstance> queryMBeansImpl = this.queryMBeansImpl(objectName, null);
            HashSet<ObjectInstance> set = new HashSet(queryMBeansImpl.size());
            for (ObjectInstance objectInstance : queryMBeansImpl) {
                try {
                    checkMBeanPermission(objectInstance.getClassName(), null, objectInstance.getObjectName(), "queryNames");
                    set.add(objectInstance);
                }
                catch (SecurityException ex) {}
            }
            Set<ObjectInstance> filterListOfObjectInstances = this.filterListOfObjectInstances(set, queryExp);
            HashSet queryNamesImpl = new HashSet(filterListOfObjectInstances.size());
            Iterator<Object> iterator2 = filterListOfObjectInstances.iterator();
            while (iterator2.hasNext()) {
                queryNamesImpl.add(((ObjectInstance)iterator2.next()).getObjectName());
            }
            return queryNamesImpl;
        }
        return this.queryNamesImpl(objectName, queryExp);
    }
    
    private Set<ObjectName> queryNamesImpl(ObjectName pattern, QueryExp query) {
        return this.objectNamesFromFilteredNamedObjects(this.repository.query(pattern, query), query);
    }
    
    @Override
    public boolean isRegistered(ObjectName nonDefaultDomain) {
        if (nonDefaultDomain == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Object name cannot be null"), "Object name cannot be null");
        }
        nonDefaultDomain = this.nonDefaultDomain(nonDefaultDomain);
        return this.repository.contains(nonDefaultDomain);
    }
    
    @Override
    public String[] getDomains() {
        if (System.getSecurityManager() != null) {
            checkMBeanPermission((String)null, null, null, "getDomains");
            String[] domains = this.repository.getDomains();
            ArrayList<Object> list = new ArrayList(domains.length);
            for (int i = 0; i < domains.length; ++i) {
                try {
                    checkMBeanPermission((String)null, null, Util.newObjectName(domains[i] + ":x=x"), "getDomains");
                    list.add(domains[i]);
                }
                catch (SecurityException ex) {}
            }
            return (String[])list.toArray(new String[list.size()]);
        }
        return this.repository.getDomains();
    }
    
    @Override
    public Integer getMBeanCount() {
        return this.repository.getCount();
    }
    
    @Override
    public Object getAttribute(ObjectName nonDefaultDomain, String str) throws MBeanException, AttributeNotFoundException, InstanceNotFoundException, ReflectionException {
        if (nonDefaultDomain == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Object name cannot be null"), "Exception occurred trying to invoke the getter on the MBean");
        }
        if (str == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Attribute cannot be null"), "Exception occurred trying to invoke the getter on the MBean");
        }
        nonDefaultDomain = this.nonDefaultDomain(nonDefaultDomain);
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "getAttribute", "Attribute = " + str + ", ObjectName = " + nonDefaultDomain);
        }
        DynamicMBean mBean = this.getMBean(nonDefaultDomain);
        checkMBeanPermission(mBean, str, nonDefaultDomain, "getAttribute");
        try {
            return mBean.getAttribute(str);
        }
        catch (AttributeNotFoundException ex) {
            throw ex;
        }
        catch (Throwable t) {
            rethrowMaybeMBeanException(t);
            throw new AssertionError();
        }
    }
    
    @Override
    public AttributeList getAttributes(ObjectName nonDefaultDomain, String[] array) throws InstanceNotFoundException, ReflectionException {
        if (nonDefaultDomain == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("ObjectName name cannot be null"), "Exception occurred trying to invoke the getter on the MBean");
        }
        if (array == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Attributes cannot be null"), "Exception occurred trying to invoke the getter on the MBean");
        }
        nonDefaultDomain = this.nonDefaultDomain(nonDefaultDomain);
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "getAttributes", "ObjectName = " + nonDefaultDomain);
        }
        DynamicMBean mBean = this.getMBean(nonDefaultDomain);
        if (System.getSecurityManager() == null) {
        }
        else {
            String className = getClassName(mBean);
            checkMBeanPermission(className, null, nonDefaultDomain, "getAttribute");
            ArrayList list = new ArrayList(array.length);
            for (String s : array) {
                try {
                    checkMBeanPermission(className, s, nonDefaultDomain, "getAttribute");
                    list.add(s);
                }
                catch (SecurityException ex) {}
            }
            array = (String[])list.toArray(new String[list.size()]);
        }
        try {
            return mBean.getAttributes(array);
        }
        catch (Throwable t) {
            rethrow(t);
            throw new AssertionError();
        }
    }
    
    @Override
    public void setAttribute(ObjectName nonDefaultDomain, Attribute attribute) throws InstanceNotFoundException, AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
        if (nonDefaultDomain == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("ObjectName name cannot be null"), "Exception occurred trying to invoke the setter on the MBean");
        }
        if (attribute == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Attribute cannot be null"), "Exception occurred trying to invoke the setter on the MBean");
        }
        nonDefaultDomain = this.nonDefaultDomain(nonDefaultDomain);
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "setAttribute", "ObjectName = " + nonDefaultDomain + ", Attribute = " + attribute.getName());
        }
        DynamicMBean mBean = this.getMBean(nonDefaultDomain);
        checkMBeanPermission(mBean, attribute.getName(), nonDefaultDomain, "setAttribute");
        try {
            mBean.setAttribute(attribute);
        }
        catch (AttributeNotFoundException ex) {
            throw ex;
        }
        catch (InvalidAttributeValueException ex2) {
            throw ex2;
        }
        catch (Throwable t) {
            rethrowMaybeMBeanException(t);
            throw new AssertionError();
        }
        return;
    }
    
    @Override
    public AttributeList setAttributes(ObjectName nonDefaultDomain, AttributeList attributes) throws InstanceNotFoundException, ReflectionException {
        if (nonDefaultDomain == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("ObjectName name cannot be null"), "Exception occurred trying to invoke the setter on the MBean");
        }
        if (attributes == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("AttributeList  cannot be null"), "Exception occurred trying to invoke the setter on the MBean");
        }
        nonDefaultDomain = this.nonDefaultDomain(nonDefaultDomain);
        DynamicMBean mBean = this.getMBean(nonDefaultDomain);
        if (System.getSecurityManager() == null) {
        }
        else {
            String className = getClassName(mBean);
            checkMBeanPermission(className, null, nonDefaultDomain, "setAttribute");
            attributes = new AttributeList(attributes.size());
            for (Attribute object : attributes.asList()) {
                try {
                    checkMBeanPermission(className, object.getName(), nonDefaultDomain, "setAttribute");
                    attributes.add(object);
                }
                catch (SecurityException ex) {}
            }
        }
        try {
            return mBean.setAttributes(attributes);
        }
        catch (Throwable t) {
            rethrow(t);
            throw new AssertionError();
        }
    }
    
    @Override
    public Object invoke(ObjectName nonDefaultDomain, String s, Object[] array, String[] array2) throws InstanceNotFoundException, MBeanException, ReflectionException {
        nonDefaultDomain = this.nonDefaultDomain(nonDefaultDomain);
        DynamicMBean mBean = this.getMBean(nonDefaultDomain);
        checkMBeanPermission(mBean, s, nonDefaultDomain, "invoke");
        try {
            return mBean.invoke(s, array, array2);
        }
        catch (Throwable t) {
            rethrowMaybeMBeanException(t);
            throw new AssertionError();
        }
    }
    
    private static void rethrow(Throwable t) throws ReflectionException {
        try {
            throw t;
        }
        catch (ReflectionException ex) {
            throw ex;
        }
        catch (RuntimeOperationsException ex2) {
            throw ex2;
        }
        catch (RuntimeErrorException ex3) {
            throw ex3;
        }
        catch (RuntimeException e) {
            throw new RuntimeMBeanException(e, e.toString());
        }
        catch (Error e2) {
            throw new RuntimeErrorException(e2, e2.toString());
        }
        catch (Throwable cause) {
            throw new RuntimeException("Unexpected exception", cause);
        }
    }
    
    private static void rethrowMaybeMBeanException(Throwable t) throws ReflectionException, MBeanException {
        if ((t instanceof MBeanException)) {
            throw (MBeanException)t;
        }
        rethrow(t);
        return;
    }
    
    private ObjectInstance registerObject(String s, Object mbean, ObjectName objectName) throws InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {
        if (mbean == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Cannot add null object"), "Exception occurred trying to register the MBean");
        }
        return this.registerDynamicMBean(s, Introspector.makeDynamicMBean(mbean), objectName);
    }
    
    private ObjectInstance registerDynamicMBean(String className, DynamicMBean dynamicMBean, ObjectName nonDefaultDomain) throws InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {
        nonDefaultDomain = this.nonDefaultDomain(nonDefaultDomain);
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "registerMBean", "ObjectName = " + nonDefaultDomain);
        }
        ObjectName objectName = preRegister(dynamicMBean, this.server, nonDefaultDomain);
        boolean n = false;
        boolean n2 = false;
        DefaultMBeanServerInterceptor.ResourceContext registerWithRepository = null;
        try {
            if ((dynamicMBean instanceof DynamicMBean2)) {
                try {
                    ((DynamicMBean2)dynamicMBean).preRegister2(this.server, objectName);
                    n2 = true;
                }
                catch (Exception cause) {
                    if ((cause instanceof RuntimeException)) {
                        throw (RuntimeException)cause;
                    }
                    if ((cause instanceof InstanceAlreadyExistsException)) {
                        throw (InstanceAlreadyExistsException)cause;
                    }
                    throw new RuntimeException(cause);
                }
            }
            if (objectName != nonDefaultDomain && objectName != null) {
                objectName = ObjectName.getInstance(this.nonDefaultDomain(objectName));
            }
            checkMBeanPermission(className, null, objectName, "registerMBean");
            if (objectName == null) {
                throw new RuntimeOperationsException(new IllegalArgumentException("No object name specified"), "Exception occurred trying to register the MBean");
            }
            registerWithRepository = this.registerWithRepository(getResource(dynamicMBean), dynamicMBean, objectName);
            n2 = false;
            n = true;
        }
        finally {
            try {
                postRegister(objectName, dynamicMBean, (boolean)n, (boolean)n2);
            }
            finally {
                if (n && registerWithRepository != null) {
                    registerWithRepository.done();
                }
            }
        }
        return new ObjectInstance(objectName, className);
    }
    
    private static void throwMBeanRegistrationException(Throwable cause, String str) throws MBeanRegistrationException {
        if ((cause instanceof RuntimeException)) {
            throw new RuntimeMBeanException((RuntimeException)cause, "RuntimeException thrown " + str);
        }
        if ((cause instanceof Error)) {
            throw new RuntimeErrorException((Error)cause, "Error thrown " + str);
        }
        if ((cause instanceof MBeanRegistrationException)) {
            throw (MBeanRegistrationException)cause;
        }
        if ((cause instanceof Exception)) {
            throw new MBeanRegistrationException((Exception)cause, "Exception thrown " + str);
        }
        throw new RuntimeException(cause);
    }
    
    private static ObjectName preRegister(DynamicMBean dynamicMBean, MBeanServer mBeanServer, ObjectName objectName) throws InstanceAlreadyExistsException, MBeanRegistrationException {
        ObjectName preRegister = null;
        try {
            if ((dynamicMBean instanceof MBeanRegistration)) {
                preRegister = ((MBeanRegistration)dynamicMBean).preRegister(mBeanServer, objectName);
            }
        }
        catch (Throwable t) {
            throwMBeanRegistrationException(t, "in preRegister method");
        }
        if (preRegister != null) {
            return preRegister;
        }
        return objectName;
    }
    
    private static void postRegister(ObjectName objectName, DynamicMBean dynamicMBean, boolean b, boolean b2) {
        if (b2 && dynamicMBean instanceof DynamicMBean2) {
            ((DynamicMBean2)dynamicMBean).registerFailed();
        }
        try {
            if ((dynamicMBean instanceof MBeanRegistration)) {
                ((MBeanRegistration)dynamicMBean).postRegister(Boolean.valueOf(b));
            }
        }
        catch (RuntimeException obj) {
            JmxProperties.MBEANSERVER_LOGGER.fine("While registering MBean [" + objectName + "]: Exception thrown by postRegister: rethrowing <" + obj + ">, but keeping the MBean registered");
            throw new RuntimeMBeanException(obj, "RuntimeException thrown in postRegister method: rethrowing <" + obj + ">, but keeping the MBean registered");
        }
        catch (Error obj2) {
            JmxProperties.MBEANSERVER_LOGGER.fine("While registering MBean [" + objectName + "]: Error thrown by postRegister: rethrowing <" + obj2 + ">, but keeping the MBean registered");
            throw new RuntimeErrorException(obj2, "Error thrown in postRegister method: rethrowing <" + obj2 + ">, but keeping the MBean registered");
        }
    }
    
    private static void preDeregisterInvoke(MBeanRegistration mBeanRegistration) throws MBeanRegistrationException {
        try {
            mBeanRegistration.preDeregister();
        }
        catch (Throwable t) {
            throwMBeanRegistrationException(t, "in preDeregister method");
        }
    }
    
    private static void postDeregisterInvoke(ObjectName objectName, MBeanRegistration mBeanRegistration) {
        try {
            mBeanRegistration.postDeregister();
        }
        catch (RuntimeException obj) {
            JmxProperties.MBEANSERVER_LOGGER.fine("While unregistering MBean [" + objectName + "]: Exception thrown by postDeregister: rethrowing <" + obj + ">, although the MBean is succesfully unregistered");
            throw new RuntimeMBeanException(obj, "RuntimeException thrown in postDeregister method: rethrowing <" + obj + ">, although the MBean is sucessfully unregistered");
        }
        catch (Error obj2) {
            JmxProperties.MBEANSERVER_LOGGER.fine("While unregistering MBean [" + objectName + "]: Error thrown by postDeregister: rethrowing <" + obj2 + ">, although the MBean is succesfully unregistered");
            throw new RuntimeErrorException(obj2, "Error thrown in postDeregister method: rethrowing <" + obj2 + ">, although the MBean is sucessfully unregistered");
        }
    }
    
    private DynamicMBean getMBean(ObjectName objectName) throws InstanceNotFoundException {
        if (objectName == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Object name cannot be null"), "Exception occurred trying to get an MBean");
        }
        DynamicMBean retrieve = this.repository.retrieve(objectName);
        if (retrieve == null) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "getMBean", objectName + " : Found no object");
            }
            throw new InstanceNotFoundException(objectName.toString());
        }
        else {
            return retrieve;
        }
    }
    
    private static Object getResource(DynamicMBean dynamicMBean) {
        if ((dynamicMBean instanceof DynamicMBean2)) {
            return ((DynamicMBean2)dynamicMBean).getResource();
        }
        return dynamicMBean;
    }
    
    private ObjectName nonDefaultDomain(ObjectName obj) {
        if (obj == null || obj.getDomain().length() > 0) {
            return obj;
        }
        return Util.newObjectName(this.domain + obj);
    }
    
    @Override
    public String getDefaultDomain() {
        return this.domain;
    }
    
    @Override
    public void addNotificationListener(ObjectName obj, NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException {
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "addNotificationListener", "ObjectName = " + obj);
        }
        DynamicMBean mBean = this.getMBean(obj);
        checkMBeanPermission(mBean, null, obj, "addNotificationListener");
        NotificationBroadcaster notificationBroadcaster = getNotificationBroadcaster(obj, mBean, NotificationBroadcaster.class);
        if (notificationListener == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Null listener"), "Null listener");
        }
        notificationBroadcaster.addNotificationListener(this.getListenerWrapper(notificationListener, obj, mBean, true), notificationFilter, o);
        return;
    }
    
    @Override
    public void addNotificationListener(ObjectName obj, ObjectName obj2, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException {
        Object resource = getResource(this.getMBean(obj2));
        if (!(resource instanceof NotificationListener)) {
            throw new RuntimeOperationsException(new IllegalArgumentException(obj2.getCanonicalName()), "The MBean " + obj2.getCanonicalName() + "does not implement the NotificationListener interface");
        }
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "addNotificationListener", "ObjectName = " + obj + ", Listener = " + obj2);
        }
        this.server.addNotificationListener(obj, (NotificationListener)resource, notificationFilter, o);
        return;
    }
    
    @Override
    public void removeNotificationListener(ObjectName objectName, NotificationListener notificationListener) throws InstanceNotFoundException, ListenerNotFoundException {
        this.removeNotificationListener(objectName, notificationListener, null, null, true);
    }
    
    @Override
    public void removeNotificationListener(ObjectName objectName, NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException, ListenerNotFoundException {
        this.removeNotificationListener(objectName, notificationListener, notificationFilter, o, false);
    }
    
    @Override
    public void removeNotificationListener(ObjectName obj, ObjectName obj2) throws InstanceNotFoundException, ListenerNotFoundException {
        NotificationListener listener = this.getListener(obj2);
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "removeNotificationListener", "ObjectName = " + obj + ", Listener = " + obj2);
        }
        this.server.removeNotificationListener(obj, listener);
    }
    
    @Override
    public void removeNotificationListener(ObjectName obj, ObjectName obj2, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException, ListenerNotFoundException {
        NotificationListener listener = this.getListener(obj2);
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "removeNotificationListener", "ObjectName = " + obj + ", Listener = " + obj2);
        }
        this.server.removeNotificationListener(obj, listener, notificationFilter, o);
    }
    
    private NotificationListener getListener(ObjectName objectName) throws ListenerNotFoundException {
        DynamicMBean mBean;
        try {
            mBean = this.getMBean(objectName);
        }
        catch (InstanceNotFoundException cause) {
            throw (ListenerNotFoundException)EnvHelp.initCause(new ListenerNotFoundException(cause.getMessage()), cause);
        }
        Object resource = getResource(mBean);
        if (!(resource instanceof NotificationListener)) {
            throw new RuntimeOperationsException(new IllegalArgumentException(objectName.getCanonicalName()), "MBean " + objectName.getCanonicalName() + " does not implement " + NotificationListener.class.getName());
        }
        return (NotificationListener)resource;
    }
    
    private void removeNotificationListener(ObjectName obj, NotificationListener notificationListener, NotificationFilter notificationFilter, Object o, boolean b) throws InstanceNotFoundException, ListenerNotFoundException {
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "removeNotificationListener", "ObjectName = " + obj);
        }
        DynamicMBean mBean = this.getMBean(obj);
        checkMBeanPermission(mBean, null, obj, "removeNotificationListener");
        NotificationBroadcaster notificationBroadcaster = getNotificationBroadcaster(obj, mBean, b ? NotificationBroadcaster.class : NotificationEmitter.class);
        NotificationListener listenerWrapper = this.getListenerWrapper(notificationListener, obj, mBean, false);
        if (listenerWrapper == null) {
            throw new ListenerNotFoundException("Unknown listener");
        }
        if (b) {
            notificationBroadcaster.removeNotificationListener(listenerWrapper);
        }
        else {
            ((NotificationEmitter)notificationBroadcaster).removeNotificationListener(listenerWrapper, notificationFilter, o);
        }
        return;
    }
    
    private static <T extends NotificationBroadcaster> T getNotificationBroadcaster(ObjectName objectName, Object resource, Class<T> clazz) {
        if (clazz.isInstance(resource)) {
            return (T)clazz.cast(resource);
        }
        if ((resource instanceof DynamicMBean2)) {
            resource = ((DynamicMBean2)resource).getResource();
        }
        if (clazz.isInstance(resource)) {
            return (T)clazz.cast(resource);
        }
        throw new RuntimeOperationsException(new IllegalArgumentException(objectName.getCanonicalName()), "MBean " + objectName.getCanonicalName() + " does not implement " + clazz.getName());
    }
    
    @Override
    public MBeanInfo getMBeanInfo(ObjectName obj) throws InstanceNotFoundException, IntrospectionException, ReflectionException {
        DynamicMBean mBean = this.getMBean(obj);
        MBeanInfo mBeanInfo;
        try {
            mBeanInfo = mBean.getMBeanInfo();
        }
        catch (RuntimeMBeanException ex) {
            throw ex;
        }
        catch (RuntimeErrorException ex2) {
            throw ex2;
        }
        catch (RuntimeException e) {
            throw new RuntimeMBeanException(e, "getMBeanInfo threw RuntimeException");
        }
        catch (Error e2) {
            throw new RuntimeErrorException(e2, "getMBeanInfo threw Error");
        }
        if (mBeanInfo == null) {
            throw new JMRuntimeException("MBean " + obj + "has no MBeanInfo");
        }
        checkMBeanPermission(mBeanInfo.getClassName(), null, obj, "getMBeanInfo");
        return mBeanInfo;
    }
    
    @Override
    public boolean isInstanceOf(ObjectName objectName, String s) throws InstanceNotFoundException {
        DynamicMBean mBean = this.getMBean(objectName);
        checkMBeanPermission(mBean, null, objectName, "isInstanceOf");
        try {
            Object resource = getResource(mBean);
            String name = (resource instanceof DynamicMBean) ? getClassName((DynamicMBean)resource) : resource.getClass().getName();
            if (name.equals(s)) {
                return true;
            }
            ClassLoader classLoader = resource.getClass().getClassLoader();
            Class<Object> forName = Class.forName(s, false, classLoader);
            return forName.isInstance(resource) || forName.isAssignableFrom(Class.forName(name, false, classLoader));
        }
        catch (Exception thrown) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINEST, DefaultMBeanServerInterceptor.class.getName(), "isInstanceOf", "Exception calling isInstanceOf", thrown);
            }
            return false;
        }
    }
    
    @Override
    public ClassLoader getClassLoaderFor(ObjectName objectName) throws InstanceNotFoundException {
        DynamicMBean mBean = this.getMBean(objectName);
        checkMBeanPermission(mBean, null, objectName, "getClassLoaderFor");
        return getResource(mBean).getClass().getClassLoader();
    }
    
    @Override
    public ClassLoader getClassLoader(ObjectName objectName) throws InstanceNotFoundException {
        if (objectName == null) {
            checkMBeanPermission((String)null, null, null, "getClassLoader");
            return this.server.getClass().getClassLoader();
        }
        DynamicMBean mBean = this.getMBean(objectName);
        checkMBeanPermission(mBean, null, objectName, "getClassLoader");
        Object resource = getResource(mBean);
        if (!(resource instanceof ClassLoader)) {
            throw new InstanceNotFoundException(objectName.toString() + " is not a classloader");
        }
        return (ClassLoader)resource;
    }
    
    private void sendNotification(String s, ObjectName objectName) {
        MBeanServerNotification notification = new MBeanServerNotification(s, MBeanServerDelegate.DELEGATE_NAME, 0L, objectName);
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "sendNotification", s + " " + objectName);
        }
        this.delegate.sendNotification(notification);
    }
    
    private Set<ObjectName> objectNamesFromFilteredNamedObjects(Set<NamedObject> set, QueryExp queryExp) {
        HashSet<ObjectName> set2 = new HashSet();
        if (queryExp == null) {
            Iterator<Object> iterator2 = set.iterator();
            while (iterator2.hasNext()) {
                set2.add(((NamedObject)iterator2.next()).getName());
            }
        }
        else {
            MBeanServer mBeanServer = QueryEval.getMBeanServer();
            queryExp.setMBeanServer(this.server);
            try {
                for (NamedObject namedObject : set) {
                    int apply;
                    try {
                        apply = queryExp.apply(namedObject.getName());
                    }
                    catch (Exception ex) {
                        apply = false;
                    }
                    if (!(apply == 0)) {
                        set2.add(namedObject.getName());
                    }
                    else {
                        continue;
                    }
                }
            }
            finally {
                queryExp.setMBeanServer(mBeanServer);
            }
        }
        return set2;
    }
    
    private Set<ObjectInstance> objectInstancesFromFilteredNamedObjects(Set<NamedObject> set, QueryExp queryExp) {
        HashSet<ObjectInstance> set2 = new HashSet();
        if (queryExp == null) {
            for (NamedObject namedObject2 : set) {
                set2.add(new ObjectInstance(namedObject2.getName(), safeGetClassName(namedObject2.getObject())));
            }
        }
        else {
            MBeanServer mBeanServer = QueryEval.getMBeanServer();
            queryExp.setMBeanServer(this.server);
            try {
                for (NamedObject namedObject : set) {
                    DynamicMBean object = namedObject.getObject();
                    int apply;
                    try {
                        apply = queryExp.apply(namedObject.getName());
                    }
                    catch (Exception ex) {
                        apply = false;
                    }
                    if (!(apply == 0)) {
                        set2.add(new ObjectInstance(namedObject.getName(), safeGetClassName(object)));
                    }
                    else {
                        continue;
                    }
                }
            }
            finally {
                queryExp.setMBeanServer(mBeanServer);
            }
        }
        return set2;
    }
    
    private static String safeGetClassName(DynamicMBean dynamicMBean) {
        try {
            return getClassName(dynamicMBean);
        }
        catch (Exception thrown) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINEST, DefaultMBeanServerInterceptor.class.getName(), "safeGetClassName", "Exception getting MBean class name", thrown);
            }
            return null;
        }
    }
    
    private Set<ObjectInstance> filterListOfObjectInstances(Set<ObjectInstance> set, QueryExp queryExp) {
        if (queryExp == null) {
            return set;
        }
        HashSet<ObjectInstance> set2 = new HashSet();
        for (ObjectInstance objectInstance : set) {
            boolean apply = false;
            MBeanServer mBeanServer = QueryEval.getMBeanServer();
            queryExp.setMBeanServer(this.server);
            try {
                apply = queryExp.apply(objectInstance.getObjectName());
            }
            catch (Exception ex) {
                apply = false;
            }
            finally {
                queryExp.setMBeanServer(mBeanServer);
            }
            if (!(apply == 0)) {
                set2.add(objectInstance);
            }
            else {
                continue;
            }
        }
        return set2;
    }
    
    private NotificationListener getListenerWrapper(NotificationListener notificationListener, ObjectName objectName, DynamicMBean dynamicMBean, boolean b) {
        DefaultMBeanServerInterceptor.ListenerWrapper referent = new DefaultMBeanServerInterceptor.ListenerWrapper(notificationListener, objectName, getResource(dynamicMBean));
        synchronized (this.listenerWrappers) {
            WeakReference<Object> weakReference = (WeakReference)this.listenerWrappers.get(referent);
            if (weakReference != null) {
                NotificationListener notificationListener2 = (NotificationListener)weakReference.get();
                if (notificationListener2 != null) {
                    return notificationListener2;
                }
            }
            if (b) {
                this.listenerWrappers.put(referent, new WeakReference(referent));
                return referent;
            }
            return null;
        }
    }
    
    @Override
    public Object instantiate(String s) throws ReflectionException, MBeanException {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    @Override
    public Object instantiate(String s, ObjectName objectName) throws ReflectionException, MBeanException, InstanceNotFoundException {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    @Override
    public Object instantiate(String s, Object[] array, String[] array2) throws ReflectionException, MBeanException {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    @Override
    public Object instantiate(String s, ObjectName objectName, Object[] array, String[] array2) throws ReflectionException, MBeanException, InstanceNotFoundException {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    @Override
    public ObjectInputStream deserialize(ObjectName objectName, byte[] array) throws InstanceNotFoundException, OperationsException {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    @Override
    public ObjectInputStream deserialize(String s, byte[] array) throws OperationsException, ReflectionException {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    @Override
    public ObjectInputStream deserialize(String s, ObjectName objectName, byte[] array) throws InstanceNotFoundException, OperationsException, ReflectionException {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    @Override
    public ClassLoaderRepository getClassLoaderRepository() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private static String getClassName(DynamicMBean dynamicMBean) {
        if ((dynamicMBean instanceof DynamicMBean2)) {
            return ((DynamicMBean2)dynamicMBean).getClassName();
        }
        return dynamicMBean.getMBeanInfo().getClassName();
    }
    
    private static void checkMBeanPermission(DynamicMBean dynamicMBean, String s, ObjectName objectName, String s2) {
        if (System.getSecurityManager() != null) {
            checkMBeanPermission(safeGetClassName(dynamicMBean), s, objectName, s2);
        }
    }
    
    private static void checkMBeanPermission(String className, String member, ObjectName objectName, String actions) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new MBeanPermission(className, member, objectName, actions));
        }
    }
    
    private static void checkMBeanTrustPermission(final Class<?> clazz) throws SecurityException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new MBeanTrustPermission("register"), new AccessControlContext(new ProtectionDomain[] { (ProtectionDomain)AccessController.doPrivileged(new PrivilegedAction<ProtectionDomain>() {
                    @Override
                    public ProtectionDomain run() {
                        return clazz.getProtectionDomain();
                    }
                }) }));
        }
    }
    
    private DefaultMBeanServerInterceptor.ResourceContext registerWithRepository(Object o, DynamicMBean object, ObjectName name) throws InstanceAlreadyExistsException, MBeanRegistrationException {
        DefaultMBeanServerInterceptor.ResourceContext resourceContext = this.makeResourceContextFor(o, name);
        this.repository.addMBean(object, name, resourceContext);
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "addObject", "Send create notification of object " + name.getCanonicalName());
        }
        this.sendNotification("JMX.mbean.registered", name);
        return resourceContext;
    }
    
    private DefaultMBeanServerInterceptor.ResourceContext unregisterFromRepository(Object o, DynamicMBean dynamicMBean, ObjectName name) throws InstanceNotFoundException {
        DefaultMBeanServerInterceptor.ResourceContext resourceContext = this.makeResourceContextFor(o, name);
        this.repository.remove(name, resourceContext);
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, DefaultMBeanServerInterceptor.class.getName(), "unregisterMBean", "Send delete notification of object " + name.getCanonicalName());
        }
        this.sendNotification("JMX.mbean.unregistered", name);
        return resourceContext;
    }
    
    private void addClassLoader(ClassLoader classLoader, ObjectName objectName) {
        ModifiableClassLoaderRepository instantiatorCLR = this.getInstantiatorCLR();
        if (instantiatorCLR == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Dynamic addition of class loaders is not supported"), "Exception occurred trying to register the MBean as a class loader");
        }
        instantiatorCLR.addClassLoader(objectName, classLoader);
        return;
    }
    
    private void removeClassLoader(ClassLoader classLoader, ObjectName objectName) {
        if (classLoader != this.server.getClass().getClassLoader()) {
            ModifiableClassLoaderRepository instantiatorCLR = this.getInstantiatorCLR();
            if (instantiatorCLR != null) {
                instantiatorCLR.removeClassLoader(objectName);
            }
        }
    }
    
    private DefaultMBeanServerInterceptor.ResourceContext createClassLoaderContext(final ClassLoader classLoader, final ObjectName objectName) {
        return new DefaultMBeanServerInterceptor.ResourceContext() {
            @Override
            public void registering() {
                DefaultMBeanServerInterceptor.access$000(DefaultMBeanServerInterceptor.this, classLoader, objectName);
            }
            
            @Override
            public void unregistered() {
                DefaultMBeanServerInterceptor.access$100(DefaultMBeanServerInterceptor.this, classLoader, objectName);
            }
            
            @Override
            public void done() {
            }
        };
    }
    
    private DefaultMBeanServerInterceptor.ResourceContext makeResourceContextFor(Object o, ObjectName objectName) {
        if ((o instanceof ClassLoader)) {
            return this.createClassLoaderContext((ClassLoader)o, objectName);
        }
        return DefaultMBeanServerInterceptor.ResourceContext.NONE;
    }
    
    private ModifiableClassLoaderRepository getInstantiatorCLR() {
        return (ModifiableClassLoaderRepository)AccessController.doPrivileged(new PrivilegedAction<ModifiableClassLoaderRepository>() {
            @Override
            public ModifiableClassLoaderRepository run() {
                return (DefaultMBeanServerInterceptor.this.instantiator != null) ? DefaultMBeanServerInterceptor.this.instantiator.getClassLoaderRepository() : null;
            }
        });
    }
    
    static void access$000(DefaultMBeanServerInterceptor defaultMBeanServerInterceptor, ClassLoader classLoader, ObjectName objectName) {
        defaultMBeanServerInterceptor.addClassLoader(classLoader, objectName);
    }
    
    static void access$100(DefaultMBeanServerInterceptor defaultMBeanServerInterceptor, ClassLoader classLoader, ObjectName objectName) {
        defaultMBeanServerInterceptor.removeClassLoader(classLoader, objectName);
    }
    
    private static class ListenerWrapper implements NotificationListener
    {
        private NotificationListener listener;
        private ObjectName name;
        private Object mbean;
        
        ListenerWrapper(NotificationListener listener, ObjectName name, Object mbean) {
            this.listener = listener;
            this.name = name;
            this.mbean = mbean;
        }
        
        @Override
        public void handleNotification(Notification notification, Object o) {
            if (notification != null && notification.getSource() == this.mbean) {
                notification.setSource(this.name);
            }
            this.listener.handleNotification(notification, o);
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof DefaultMBeanServerInterceptor.ListenerWrapper)) {
                return false;
            }
            DefaultMBeanServerInterceptor.ListenerWrapper listenerWrapper = (DefaultMBeanServerInterceptor.ListenerWrapper)o;
            return listenerWrapper.listener == this.listener && listenerWrapper.mbean == this.mbean && listenerWrapper.name.equals(this.name);
        }
        
        @Override
        public int hashCode() {
            return System.identityHashCode(this.listener) ^ System.identityHashCode(this.mbean);
        }
    }
    
    private interface ResourceContext extends Repository.RegistrationContext
    {
        public static final DefaultMBeanServerInterceptor.ResourceContext NONE = new DefaultMBeanServerInterceptor.ResourceContext() {
            @Override
            public void done() {
            }
            
            @Override
            public void registering() {
            }
            
            @Override
            public void unregistered() {
            }
        };
        
        public abstract void done();
    }
}
